/*
    This is a somewhat contrived program containing an example of a
    tail recursive procedure that cannot be tail recursion optimized.

    The idea of the program is to create a linked list of "random nodes",
    each node of which contains a random integer between 0 and 9, and
    see how long the list gets before the nodes sum up to 100. The
    procedure create_nodes() puts a new node at the head of the list,
    sums the list, and then checks if the sum has reached 100. If not,
    create_nodes() calls itself tail recursively to put another node at
    the head of the linked list.
*/
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#define TOTAL 100  // Increase the value of TOTAL to make
                   // this program overflow its stack.

typedef struct Random_node
{  int i;
   struct Random_node* next_ptr;
} Random_node;


int sum_nodes( Random_node* node_ptr )
{
   int total = (*node_ptr).i;
   while( (*node_ptr).next_ptr != NULL )
   {
      node_ptr = (*node_ptr).next_ptr;
      total += (*node_ptr).i;
   }
   return total;
}


int create_nodes( Random_node* node_ptr, int counter )
{
   // create a new node
   Random_node new_node;

   // put the new node at the head of the linked list
   new_node.next_ptr = node_ptr;
   new_node.i = rand() % 10;
   counter++;

   // sum the values in the linked list's nodes
   if ( sum_nodes(&new_node) >= TOTAL )
     return counter;
   else
     return create_nodes( &new_node, counter );  // tail recursion
}


int main()
{
   int count;

   // create the tail node for the linked list of nodes
   Random_node tail;
   tail.next_ptr = NULL;
   tail.i = rand() % 10;

   srand( time(NULL) );  // seed the random number generator

   // start adding nodes to the linked list
   count = create_nodes( &tail, 1 );
   printf("It took %d nodes to sum to 100.", count);

   return 0;
}
